pub struct JsBuilder<'a> { /* private fields */ }Expand description
Main JavaScript code builder
Wraps OXC’s AstBuilder and provides ergonomic methods for building AST nodes. All AST nodes created by this builder share the same allocator lifetime.
Examples
use fob_gen::JsBuilder;
use oxc_allocator::Allocator;
let allocator = Allocator::default();
let js = JsBuilder::new(&allocator);
// Create a simple variable declaration
let stmt = js.const_decl("message", js.string("Hello, world!"));Implementations
Sourceimpl<'a> JsBuilder<'a>
impl<'a> JsBuilder<'a>
Sourcepub fn ast(&self) -> &AstBuilder<'a>
pub fn ast(&self) -> &AstBuilder<'a>
Get the underlying AstBuilder (for advanced usage)
Sourcepub fn ident(&self, name: impl Into<Atom<'a>>) -> Expression<'a>
pub fn ident(&self, name: impl Into<Atom<'a>>) -> Expression<'a>
Create an identifier expression: name
Sourcepub fn string(&self, value: impl Into<Atom<'a>>) -> Expression<'a>
pub fn string(&self, value: impl Into<Atom<'a>>) -> Expression<'a>
Create a string literal: "value"
Sourcepub fn number(&self, value: f64) -> Expression<'a>
pub fn number(&self, value: f64) -> Expression<'a>
Create a number literal: 42
Sourcepub fn null(&self) -> Expression<'a>
pub fn null(&self) -> Expression<'a>
Create null: null
Sourcepub fn bool(&self, value: bool) -> Expression<'a>
pub fn bool(&self, value: bool) -> Expression<'a>
Create boolean: true or false
Sourcepub fn member(
&self,
object: Expression<'a>,
property: impl Into<Atom<'a>>,
) -> Expression<'a>
pub fn member( &self, object: Expression<'a>, property: impl Into<Atom<'a>>, ) -> Expression<'a>
Create a member expression: obj.prop
Sourcepub fn computed_member(
&self,
object: Expression<'a>,
index: Expression<'a>,
) -> Expression<'a>
pub fn computed_member( &self, object: Expression<'a>, index: Expression<'a>, ) -> Expression<'a>
Create a computed member expression: obj[expr]
This is critical for array indexing: arr[0], pageRoutes[idx], etc.
Sourcepub fn call(
&self,
callee: Expression<'a>,
args: Vec<Argument<'a>>,
) -> Expression<'a>
pub fn call( &self, callee: Expression<'a>, args: Vec<Argument<'a>>, ) -> Expression<'a>
Create a call expression: callee(args...)
Sourcepub fn arg(&self, expr: Expression<'a>) -> Argument<'a>
pub fn arg(&self, expr: Expression<'a>) -> Argument<'a>
Create an argument from an expression
Sourcepub fn object(&self, props: Vec<ObjectPropertyKind<'a>>) -> Expression<'a>
pub fn object(&self, props: Vec<ObjectPropertyKind<'a>>) -> Expression<'a>
Create an object expression: { key1: value1, key2: value2 }
Sourcepub fn prop(
&self,
key: impl Into<Atom<'a>>,
value: Expression<'a>,
) -> ObjectPropertyKind<'a>
pub fn prop( &self, key: impl Into<Atom<'a>>, value: Expression<'a>, ) -> ObjectPropertyKind<'a>
Create an object property: key: value
Sourcepub fn array(&self, elements: Vec<Expression<'a>>) -> Expression<'a>
pub fn array(&self, elements: Vec<Expression<'a>>) -> Expression<'a>
Create an array expression: [elem1, elem2, ...]
Sourcepub fn arrow_fn(
&self,
params: Vec<&'a str>,
body: Expression<'a>,
) -> Expression<'a>
pub fn arrow_fn( &self, params: Vec<&'a str>, body: Expression<'a>, ) -> Expression<'a>
Create an arrow function with expression body: (params) => expr
Sourcepub fn arrow_fn_block(
&self,
params: Vec<&'a str>,
stmts: Vec<Statement<'a>>,
) -> Expression<'a>
pub fn arrow_fn_block( &self, params: Vec<&'a str>, stmts: Vec<Statement<'a>>, ) -> Expression<'a>
Create an arrow function with block body: (params) => { stmts }
Sourcepub fn const_decl(
&self,
name: impl Into<Atom<'a>>,
init: Expression<'a>,
) -> Statement<'a>
pub fn const_decl( &self, name: impl Into<Atom<'a>>, init: Expression<'a>, ) -> Statement<'a>
Create a const declaration: const name = init;
Sourcepub fn let_decl(
&self,
name: impl Into<Atom<'a>>,
init: Option<Expression<'a>>,
) -> Statement<'a>
pub fn let_decl( &self, name: impl Into<Atom<'a>>, init: Option<Expression<'a>>, ) -> Statement<'a>
Create a let declaration: let name = init;
Sourcepub fn expr_stmt(&self, expr: Expression<'a>) -> Statement<'a>
pub fn expr_stmt(&self, expr: Expression<'a>) -> Statement<'a>
Create an expression statement
Sourcepub fn if_stmt(
&self,
test: Expression<'a>,
consequent: Vec<Statement<'a>>,
alternate: Option<Vec<Statement<'a>>>,
) -> Statement<'a>
pub fn if_stmt( &self, test: Expression<'a>, consequent: Vec<Statement<'a>>, alternate: Option<Vec<Statement<'a>>>, ) -> Statement<'a>
Create an if statement: if (test) { consequent } else { alternate }
Sourcepub fn return_stmt(&self, expr: Option<Expression<'a>>) -> Statement<'a>
pub fn return_stmt(&self, expr: Option<Expression<'a>>) -> Statement<'a>
Create a return statement: return expr;
Sourcepub fn throw(&self, expr: Expression<'a>) -> Statement<'a>
pub fn throw(&self, expr: Expression<'a>) -> Statement<'a>
Create a throw statement: throw expr;
Sourcepub fn import_default(
&self,
local: impl Into<Atom<'a>>,
source: impl Into<Atom<'a>>,
) -> ModuleDeclaration<'a>
pub fn import_default( &self, local: impl Into<Atom<'a>>, source: impl Into<Atom<'a>>, ) -> ModuleDeclaration<'a>
Create default import: import local from 'source';
Sourcepub fn import_side_effect(
&self,
source: impl Into<Atom<'a>>,
) -> ModuleDeclaration<'a>
pub fn import_side_effect( &self, source: impl Into<Atom<'a>>, ) -> ModuleDeclaration<'a>
Create side-effect import: import 'source';
Sourcepub fn import_named(
&self,
names: Vec<impl Into<Atom<'a>>>,
source: impl Into<Atom<'a>>,
) -> ModuleDeclaration<'a>
pub fn import_named( &self, names: Vec<impl Into<Atom<'a>>>, source: impl Into<Atom<'a>>, ) -> ModuleDeclaration<'a>
Create named imports: import { name1, name2 } from 'source';
Sourcepub fn export_default(&self, expr: Expression<'a>) -> ModuleDeclaration<'a>
pub fn export_default(&self, expr: Expression<'a>) -> ModuleDeclaration<'a>
Create default export: export default expr;
Sourcepub fn export_const(
&self,
name: impl Into<Atom<'a>>,
init: Expression<'a>,
) -> ModuleDeclaration<'a>
pub fn export_const( &self, name: impl Into<Atom<'a>>, init: Expression<'a>, ) -> ModuleDeclaration<'a>
Create named export of variable: export const name = init;
Sourcepub fn program(&self, statements: Vec<Statement<'a>>) -> Result<String>
pub fn program(&self, statements: Vec<Statement<'a>>) -> Result<String>
Generate a complete program from statements
Sourcepub fn program_with_format(
&self,
statements: Vec<Statement<'a>>,
_opts: &FormatOptions,
) -> Result<String>
pub fn program_with_format( &self, statements: Vec<Statement<'a>>, _opts: &FormatOptions, ) -> Result<String>
Generate a complete program from statements with formatting options
Sourcepub fn not(&self, expr: Expression<'a>) -> Expression<'a>
pub fn not(&self, expr: Expression<'a>) -> Expression<'a>
Unary not operator: !expr
Sourcepub fn binary(
&self,
left: Expression<'a>,
op: BinaryOperator,
right: Expression<'a>,
) -> Expression<'a>
pub fn binary( &self, left: Expression<'a>, op: BinaryOperator, right: Expression<'a>, ) -> Expression<'a>
Binary expression: left op right
Sourcepub fn logical(
&self,
left: Expression<'a>,
op: LogicalOperator,
right: Expression<'a>,
) -> Expression<'a>
pub fn logical( &self, left: Expression<'a>, op: LogicalOperator, right: Expression<'a>, ) -> Expression<'a>
Logical expression: left && right or left || right
Sourcepub fn conditional(
&self,
test: Expression<'a>,
consequent: Expression<'a>,
alternate: Expression<'a>,
) -> Expression<'a>
pub fn conditional( &self, test: Expression<'a>, consequent: Expression<'a>, alternate: Expression<'a>, ) -> Expression<'a>
Conditional/ternary expression: test ? consequent : alternate
Sourcepub fn new_expr(
&self,
callee: Expression<'a>,
args: Vec<Argument<'a>>,
) -> Expression<'a>
pub fn new_expr( &self, callee: Expression<'a>, args: Vec<Argument<'a>>, ) -> Expression<'a>
New expression: new Ctor(args)
Sourcepub fn template_literal(
&self,
parts: Vec<impl Into<Atom<'a>>>,
expressions: Vec<Expression<'a>>,
) -> Expression<'a>
pub fn template_literal( &self, parts: Vec<impl Into<Atom<'a>>>, expressions: Vec<Expression<'a>>, ) -> Expression<'a>
Sourcepub fn spread(&self, expr: Expression<'a>) -> SpreadElement<'a>
pub fn spread(&self, expr: Expression<'a>) -> SpreadElement<'a>